;;; Directory Local Variables
;;; For more information see (info "(emacs) Directory Variables")

((emacs-lisp-mode
  (bug-reference-url-format . "https://github.com/abo-abo/hydra/issues/%s")
  (indent-tabs-mode . nil)))
